@@ -24,6 +24,19 @@ from pygments.token import Generic, Token
 
 from purdy.animation import steps as steplib
 from purdy.parser import CodePart, CodeLine, LEXERS, parse_source, token_is_a
+from purdy.scribe import range_set_to_list
+
+# =============================================================================
+
+def condense(value):
+    if not value:
+        return ''
+
+    content = deepcopy(value)
+    if len(content) >= 20:
+        content = content[0:17] + '...'
+
+    return content
 
 # =============================================================================
 # Single Code Blob Actions
@@ -50,6 +63,10 @@ class Insert:
         self.code = code
         self.position = position
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.Insert({self.position}, "{content}")'
+
     def steps(self):
         lines = parse_source(self.code.source, self.code.lexer)
         steps = [steplib.InsertRows(self.code_box, self.position, lines), ]
@@ -70,6 +87,10 @@ class Append(Insert):
     def __init__(self, code_box, code):
         super().__init__(code_box, 0, code)
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.Append("{content}")'
+
 
 class Replace:
     """Replaces one or more lines of a :class:`purdy.ui.CodeBox` using the
@@ -89,6 +110,10 @@ class Replace:
         self.code = code
         self.position = position
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.Replace({self.position}, "{content}")'
+
     def steps(self):
         lines = parse_source(self.code.source, self.code.lexer)
         steps = [steplib.ReplaceRows(self.code_box, self.position, lines), ]
@@ -112,6 +137,9 @@ class Remove:
         self.position = position
         self.size = size
 
+    def __str__(self):
+        return f'actions.Remove({self.position}, "{self.size}")'
+
     def steps(self):
         return [steplib.RemoveRows(self.code_box, self.position, self.size), ]
 
@@ -125,6 +153,9 @@ class Clear:
     def __init__(self, code_box):
         self.code_box = code_box
 
+    def __str__(self):
+        return 'actions.Clear()'
+
     def steps(self):
         return [steplib.Clear(self.code_box), ]
 
@@ -149,6 +180,9 @@ class Suffix:
         self.position = position
         self.source = source
 
+    def __str__(self):
+        return f'actions.Suffix({self.position}, "{self.source}")'
+
     def steps(self):
         return [steplib.SuffixRow(self.code_box, self.position, self.source), ]
 
@@ -169,6 +203,9 @@ class Shell:
         self.code_box = code_box
         self.cmd = cmd
 
+    def __str__(self):
+        return f'actions.Suffix("{self.cmd}")'
+
     def steps(self):
         return [steplib.Subprocess(self.code_box, self.cmd), ]
 
@@ -292,6 +329,9 @@ class AppendTypewriter(TypewriterStep):
         self.code = code
         self.position = 0
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.AppendTypewriter("{content}")'
 
 class InsertTypewriter(TypewriterStep):
     """Inserts the contents of a :class:`purdy.content.Code` object at the
@@ -311,6 +351,9 @@ class InsertTypewriter(TypewriterStep):
         self.code = code
         self.position = position
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.InsertTypewriter({self.position}, "{content}")'
 
 class ReplaceTypewriter(TypewriterStep):
     """Replaces one or more lines in a :class:`CodeBox` with the contents of a 
@@ -330,6 +373,10 @@ class ReplaceTypewriter(TypewriterStep):
         self.code = code
         self.position = position
 
+    def __str__(self):
+        content = condense(self.code.source)
+        return f'actions.ReplaceTypewriter({self.position}, "{content}")'
+
     def steps(self):
         # Override TypewriterStep, need to remove the line before inserting a
         # new one
@@ -366,6 +413,9 @@ class SuffixTypewriter(TypewriterBase):
         self.position = position
         self.source = source
 
+    def __str__(self):
+        return f'actions.SuffixTypewriter({self.position}, "{self.source}")'
+
     def steps(self):
         steps = []
         for count, letter in enumerate(self.source):
@@ -401,16 +451,59 @@ class Highlight:
         self.highlight_on = highlight_on
 
         if isinstance(spec, str):
-            from purdy.scribe import range_set_to_list
-
             self.numbers = range_set_to_list(spec)
         else:
             self.numbers = spec
 
+    def __str__(self):
+        return f'actions.Highlight({self.numbers}, {self.highlight_on})'
+
     def steps(self):
         return [steplib.HighlightLines(self.code_box, self.numbers,
             self.highlight_on) ]
 
+
+class HighlightChain:
+    """A common pattern with highlighting lines is to turn a highlight on for
+    some set of lines, then turn it off and turn it on for more lines. This is
+    a convenience wrapper to the :class:`Highlight` action, turning items on
+    and off in series.
+
+    :param code_box: :class:`purdy.ui.CodeBox` to perform series of highlight on
+
+    :param spec_list: a list of highlight specs (see :class:`Highlight` for
+                      details on a spec)
+    """
+    def __init__(self, code_box, spec_list):
+        self.code_box = code_box
+        self.spec_list = spec_list
+
+    def __str__(self):
+        return f'actions.Highlight({self.spec_list}, {self.highlight_on})'
+
+    def steps(self):
+        all_steps = []
+        previous_numbers = None
+        for spec in self.spec_list:
+            if isinstance(spec, str):
+                numbers = range_set_to_list(spec)
+            else:
+                numbers = spec
+
+            if previous_numbers:
+                all_steps.append( steplib.HighlightLines(self.code_box, 
+                    previous_numbers, False) )
+
+            all_steps.extend([ 
+                steplib.HighlightLines(self.code_box, numbers, True),
+                steplib.CellEnd()
+            ])
+
+            previous_numbers = numbers
+
+        return all_steps
+
+
 class Fold:
     """Folds code by replacing one or more lines with a vertical elipses
     symbol.
@@ -428,6 +521,9 @@ class Fold:
         self.position = position
         self.end = end
 
+    def __str__(self):
+        return f'actions.Fold({self.position}, {self.end})'
+
     def steps(self):
         return [steplib.FoldLines(self.code_box, self.position, self.end)]
 
@@ -439,12 +535,18 @@ class Wait:
     """Causes the animations to wait for a `right arrow` key press before 
     continuing.
     """
+    def __str__(self):
+        return 'actions.Wait()'
+
     def steps(self):
         return [steplib.CellEnd(), ]
 
 
 class StopMovie:
     """Causes the presentation :class:`purdy.ui.Screen` to exit movie mode"""
+    def __str__(self):
+        return 'actions.StopMovie()'
+
     def steps(self):
         return [steplib.StopMovie(), ]
 
@@ -457,14 +559,34 @@ class Transition:
                      transition on
 
     :param code: a :class:`purdy.content.Code` object containing the source
-                 code replacing the existing content
+                 code replacing the existing content. Should not be used at
+                 the same time as code_box_to_copy parameter.
+
+    :param code_box_to_copy: a code box containing rendered code to copy into
+                             this one to display. This is typically a 
+                             :class:`VirtualCodeBox`. Should not be used at
+                             the same time as code parameter.
     """
-    def __init__(self, code_box, code):
+    def __init__(self, code_box, code=None, code_box_to_copy=None):
         self.code_box = code_box
         self.code = code
+        self.code_box_to_copy = code_box_to_copy
+
+    def __str__(self):
+        code_content = ''
+        if self.code:
+            code_content = condense(self.code.source)
+
+        copy_content = ''
+        if self.code_box_to_copy:
+            lines = [str(line) for line in self.code_box_to_copy.listing.lines]
+            copy_content = condense('\n'.join(lines))
+
+        return f'actions.Transition("{code_content}", "{copy_content}")'
 
     def steps(self):
-        return [steplib.Transition(self.code_box, self.code), ]
+        return [steplib.Transition(self.code_box, self.code, 
+            self.code_box_to_copy), ]
 
 
 class Sleep:
@@ -478,5 +600,8 @@ class Sleep:
     def __init__(self, time):
         self.time = time
 
+    def __str__(self):
+        return 'actions.Sleep'
+
     def steps(self):
         return [steplib.Sleep(self.time), ]
