@@ -15,9 +15,6 @@ from enum import Enum
 from purdy.animation import steps as steplib
 from purdy.parser import BlankCodeLine, parse_source
 
-#import logging
-#logger = logging.getLogger()
-
 # ===========================================================================
 # Cell Factory
 # ===========================================================================
@@ -32,17 +29,26 @@ def group_steps_into_cells(steps):
     """
     cells = []
     cell = None
+    previous_step = None
     for step in steps:
         if isinstance(step, steplib.CellEnd):
             if cell:
                 cells.append(cell)
 
+            if isinstance(previous_step, steplib.Transition):
+                # By default Transition automatically goes to next step, a
+                # wait is ignored because Transition itself is a cell, in this
+                # case need to change the behaviour of Transition
+                cells[-1].auto_forward = False
+
             cell = None
         elif isinstance(step, steplib.Transition):
             if cell:
                 cells.append(cell)
 
-            cell = TransitionCell(step.code_box, step.code)
+            cell = TransitionCell(step.code_box, step.code,
+                step.code_box_to_copy)
+
             cells.append(cell)
             cell = None
         else:
@@ -51,6 +57,8 @@ def group_steps_into_cells(steps):
 
             cell.steps.append(step)
 
+        previous_step = step
+
     if cell and len(cell.steps) > 0:
         cells.append(cell)
 
@@ -97,15 +105,6 @@ class GroupCell(AnimatingCellBase):
         step_string = ','.join([str(step) for step in self.steps])
         return 'GroupCell(steps=[' + step_string + '])'
 
-    def _test_dict(self):
-        d = {
-            'GroupCell': {
-                'steps':[step._test_dict() for step in self.steps],
-            }
-        }
-
-        return d
-
     def render(self, manager, skip=False):
         if len(self.steps) == 0 or self.index >= len(self.steps):
             # badly formed action sequences that result in an empty cell would
@@ -154,7 +153,6 @@ class GroupCell(AnimatingCellBase):
 class TransitionCell(AnimatingCellBase):
     DELAY = 0.05
     BETWEEN_DELAY = 0.3
-    auto_forward = True
 
     class State(Enum):
         BEFORE = 0
@@ -162,20 +160,13 @@ class TransitionCell(AnimatingCellBase):
         APPENDING = 2
         DONE = 3
 
-    def __init__(self, code_box, code):
+    def __init__(self, code_box, code, code_box_to_copy):
         self.code_box = code_box
         self.code = code
+        self.code_box_to_copy = code_box_to_copy
         self.state = self.State.BEFORE
         self.steps = []
-
-    def _test_dict(self):
-        d = {
-            'TransitionCell': {
-                'code':f'{self.code.source}',
-            }
-        }
-
-        return d
+        self.auto_forward = True
 
     def render(self, manager, skip=False):
         if self.state == self.State.DONE:
@@ -186,13 +177,17 @@ class TransitionCell(AnimatingCellBase):
         if self.state == self.State.BEFORE:
             # first time through, setup our append and undo lines
             self.position = 1
-            self.code_lines = parse_source(self.code.source, self.code.lexer)
-            self.undo_lines = deepcopy(self.code_box.listing.lines)
 
+            if self.code:
+                self.code_lines = parse_source(self.code.source, 
+                    self.code.lexer)
+            elif self.code_box_to_copy:
+                self.code_lines = deepcopy(self.code_box_to_copy.listing.lines)
+
+            self.undo_lines = deepcopy(self.code_box.listing.lines)
             self.state = self.State.DELETING
             # intentional fall through
 
-
         # If we're in skip mode, need to keep doing the work
         while True:
             if self.state == self.State.DELETING:
@@ -213,7 +208,8 @@ class TransitionCell(AnimatingCellBase):
                 except IndexError:
                     manager.state = manager.State.ACTIVE
                     self.state = self.State.DONE
-                    manager.screen.set_alarm('auto_forward_alarm', 0)
+                    if self.auto_forward:
+                        manager.screen.set_alarm('auto_forward_alarm', 0)
                     return
 
             if not skip:
