@@ -1,3 +1,11 @@
+"""
+Settings (purdy.settings.py)
+============================
+
+Defines the default settings for :class:`purdy.ui.Screen` objects, can be
+overridden by passing an altered dictionary into the Screen's constructor. 
+"""
+
 settings = {
     # delay between characters appearing on screen (in milliseconds)
     'delay':130,
@@ -11,6 +19,9 @@ settings = {
     # (like 'delay' field
     'movie_mode':-1,
 
-    # xterm colour mode, anything but 256 gives default
+    # xterm colour mode, anything but 256 gives 16 colour mode
     'colour':256,
+
+    # if True, stops Screen from running argparse
+    'deactivate_args':False,
 }
