@@ -16,6 +16,10 @@ from purdy.cmd import background_arg
 from purdy.content import Listing
 from purdy.settings import settings as default_settings
 
+#import logging
+#logging.basicConfig(filename='debug.log', level=logging.DEBUG)
+#logger = logging.getLogger()
+
 # =============================================================================
 
 DESCRIPTION = """You can alter the behaviour of any script calling
@@ -24,49 +28,101 @@ Screen.run() by passing in command line arguments."""
 class Factory:
     name = 'tui'
     iscreen = None
-    parms = {}
 
     @classmethod
-    def get_iscreen(cls, parent_screen):
-        if cls.iscreen:
-            return cls.iscreen
-
+    def get_iscreen_class(cls):
         if cls.name == 'tui':
             from purdy.iscreen.tui.iscreen import TUIScreen
-            cls.iscreen = TUIScreen(parent_screen, **cls.parms)
-            return cls.iscreen
-        else:
+            return TUIScreen
+        elif cls.name == 'virtual':
             from purdy.iscreen.virtual.iscreen import VirtualScreen
-            cls.iscreen = VirtualScreen(parent_screen, **cls.parms)
-            return cls.iscreen
+            return VirtualScreen
+        elif cls.name == 'exercise':
+            from purdy.iscreen.exercise.iscreen import ExerciseScreen
+            return ExerciseScreen
 
 # =============================================================================
 # Screen
 # =============================================================================
 
+class VirtualCodeWidget:
+    """Render hook implementation used with the :class:`VirtualCodeBox`.
+    """
+    def __init__(self):
+        self.lines = []
+
+    #--- RenderHook methods
+    def line_inserted(self, listing, position, line):
+        content = listing.render_line(line)
+        index = position - 1
+        self.lines.insert(index, content)
+
+    def line_removed(self, listing, position):
+        del self.lines[position - 1]
+
+    def line_changed(self, listing, position, line):
+        content = listing.render_line(line)
+        index = position - 1
+        self.lines[index] = content
+
+    def clear(self):
+        self.lines = []
+
+
+class VirtualCodeBox:
+    """Specifies a box to contain code. This implementation is not meant to be
+    associated with a :class:`Screen`. Used for "rendering" off screen,
+    manipulating code either for export or copying to a concrete code box
+    later
+
+    :param starting_line_number: -1 means no line numbers, anything larger will 
+                                 be the first line number displayed.  Defaults 
+                                 to -1
+    """
+    def __init__(self, starting_line_number=-1, display_mode='plain'):
+        self.starting_line_number = starting_line_number
+        self.listing = Listing(starting_line_number=starting_line_number)
+        self.widget = VirtualCodeWidget()
+
+        self.listing.set_display(display_mode, self.widget)
+
+    def perform_actions(self, actions):
+        """Performs the list of actions on the code in the box. Done separtely
+        from the main event loop so it has no interactions like Wait,
+        typewriters, etc. Everything is done in fast-forward mode.
+
+        :param actions: list of actions to perfrom on the code in this box
+        """
+        for action in actions:
+            for step in action.steps():
+                step.render_step()
+
+# -----------------------------------------------------------------------------
+
 class CodeBox:
     """Specifies a box to contain code. :class:`Screen` uses these to
     determine widget layout, subsequent actions are done within the context of 
     this box. When :func:`CodeBox.build` is called by a :class:`Screen` class
     a widget is built and this box is added to the screen.
+
+    :param starting_line_number: -1 means no line numbers, anything larger will 
+                                 be the first line number displayed.  Defaults 
+                                 to -1
+
+    :param auto_scroll: When True, :class:`purdy.widgets.CodeBox` created by 
+                        this specification automatically scrolls to newly added 
+                        content.  Defaults to True.
+
+    :param height: Number of lines the row containing this box should be.  A 
+                   value of 0 indicates automatic spacing.  Defaults to 0.
+
+    :param compact: if False, the dividing line between this box and the next 
+                    has a 1-line empty boundary. Parameter is ignored if there 
+                    is no item after this one in the rows=[] listing. Defaults 
+                    to False
     """
     def __init__(self, starting_line_number=-1, auto_scroll=True, height=0, 
             compact=False):
-        """Constructor
-
-        :param starting_line_number: -1 means no line numbers, anything larger
-                                     will be the first line number displayed.
-                                     Defaults to -1
-        :param auto_scroll: When True, :class:`purdy.widgets.CodeBox` created
-                            by this specification automatically scrolls to
-                            newly added content.  Defaults to True.
-        :param height: Number of lines the row containing this box should be. 
-                       A value of 0 indicates automatic spacing.  Defaults to 0.
-        :param compact: if False, the dividing line between this box and the 
-                        next has a 1-line empty boundary. Parameter is ignored
-                        if there is no item after this one in the rows=[]
-                        listing. Defaults to False
-        """
         self.starting_line_number = starting_line_number
         self.auto_scroll = auto_scroll
         self.height = height
@@ -236,13 +292,6 @@ class Screen:
         class Dummy:
             pass
 
-        if self.settings['deactivate_args']:
-            self.args = Dummy()
-            self.args.debugsteps = False
-            self.args.export = False
-            self.args.export_rtf = False
-            return
-
         # Screen can be influenced by command line arguments
         parser = argparse.ArgumentParser(description=DESCRIPTION)
         parser.add_argument('--debugsteps', '--ds', action='store_true', 
@@ -254,9 +303,17 @@ class Screen:
         parser.add_argument('--exportrtf', action='store_true', 
             help='Print out the results of the actions in RTF format')
 
+        parser.add_argument('--maxheight', help=('Sets a maximum screen '
+            'height for the TUI screen viewer. Ignored if not in TUI mode.'))
+
         background_arg(parser)
 
-        self.args = parser.parse_args()
+        if self.settings['deactivate_args']:
+            # Ignore whatever was passed in on command line because of the
+            # settings
+            self.args = parser.parse_args([])
+        else:
+            self.args = parser.parse_args()
 
     def load_actions(self, actions):
         steps = []
@@ -277,16 +334,14 @@ class Screen:
         exits."""
         #logger.debug(55*'=')
         self._get_args()
+        self.actions = actions
  
-        if self.args.export:
+        if self.args.export or self.args.exportrtf:
             Factory.name = 'virtual'
 
-        if self.args.exportrtf:
-            Factory.name = 'virtual'
-            Factory.parms = {'rtf':True}
-
         # Create concrete instances of iscreen an boxes
-        self.iscreen = Factory.get_iscreen(self)
+        klass = Factory.get_iscreen_class()
+        self.iscreen = klass(self)
         for row in self.rows:
             row.build(self)
 
